__author__ = "Lukas Leufen"
__date__ = '2019-11-14'


import logging
from src.helpers import TimeTracking
from src import helpers
import argparse
import time


formatter = "%(asctime)s - %(levelname)s: %(message)s  [%(filename)s:%(funcName)s:%(lineno)s]"
logging.basicConfig(level=logging.INFO, format=formatter)


class run(object):
    """
    basic run class to measure execution time. Either call this class calling it by 'with' or delete the class instance
    after finishing the measurement. The duration result is logged.
    """

    def __init__(self):
        self.time = TimeTracking()
        logging.info(f"{self.__class__.__name__} started")

    def __del__(self):
        self.time.stop()
        logging.info(f"{self.__class__.__name__} finished after {self.time}")

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def do_stuff(self):
        time.sleep(2)


class ExperimentSetup:

    def __init__(self):
        self.data_path = None
        self.experiment_path = None
        self.experiment_name = None
        self.trainable = False
        self.setup_experiment()

    def setup_experiment(self):

        # set data path of this experiment
        self.data_path = helpers.prepare_host()

        # set experiment name
        experiment_date = parser.parse_args(["experiment_date"])
        self.experiment_name, self.experiment_path = helpers.set_experiment_name(experiment_date=experiment_date)

        # set if model shall be trained or not
        self.trainable = True


class PreProcessing(run):

    def __init__(self, setup):
        super().__init__()
        self.setup = setup


class Training(run):

    def __init__(self, setup):
        super().__init__()
        self.setup = setup


class PostProcessing(run):

    def __init__(self, setup):
        super().__init__()
        self.setup = setup


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('experiment date', metavar='exp_date', type=str, nargs=1,  help='set experiment date as string',
                        default=None)

    with run():
        exp_setup = ExperimentSetup()

        PreProcessing(exp_setup)

        Training(exp_setup)

        PostProcessing(exp_setup)
